package com.hzya.frame.plugin.lets.dingtalk.callback.service.impl;

import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponseBody;
import com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceResponse;
import com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceResponseBody;
import com.hzya.frame.plugin.lets.dingtalk.callback.service.CallBackService;
import com.hzya.frame.plugin.lets.dingtalk.dao.IDingU8cBillDao;
import com.hzya.frame.plugin.lets.dingtalk.entity.DingU8cBillEntity;
import com.hzya.frame.plugin.lets.dingtalk.utils.DingTalkUtils;
import com.hzya.frame.plugin.lets.dingtalk.vo.DingCallbackCrypto;
import com.hzya.frame.plugin.lets.u8cdto.ArapDjzb;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service("callBackService")
public class CallBackServiceImpl implements CallBackService {

    protected static Logger logger = LogManager.getLogger(CallBackServiceImpl.class);

    @Autowired
    private IDingU8cBillDao dingU8cBillDao;

    @Value("${DING.CorpId}")
    private String CorpId;
    @Value("${DING.APIToken}")
    private String APIToken;

    @Value("${DING.APPKEY}")
    private String APPKEY;
    @Value("${DING.AppSecret}")
    private String AppSecret;

    @Value("${DING.call_back_token}")
    private String token;
    @Value("${DING.call_back_aes_key}")
    private String aes_key;
    @Value("${DING.call_back_appkey}")
    private String corpId;

    @Override
    public Map<String, String> saveDingTalkCallBackRes(String msg_signature, String timeStamp, String nonce, JSONObject json) {
        logger.info("=============================================================================================================================钉钉回调执行快开始》》》》》》》》》》》》》》》》》》》》》》》》》》》》》》》》》");
        try {

            // 1. 从http请求中获取加解密参数
            // 2. 使用加解密类型
            // 2、调用订阅事件接口订阅的事件为企业级事件推送，此时OWNER_KEY为：开发者后台应用的Client ID（原企业内部应用 appKey )
            //token、ase_key均为钉钉开放平台后台管理，应用设置的事件订阅,http推送中设置的参数。
            DingCallbackCrypto callbackCrypto = new DingCallbackCrypto(token, aes_key, corpId);
            String encryptMsg = json.getString("encrypt");
            String decryptMsg = callbackCrypto.getDecryptMsg(msg_signature, timeStamp, nonce, encryptMsg);

            // 3. 反序列化回调事件json数据
            JSONObject eventJson = JSON.parseObject(decryptMsg);

            String eventType = eventJson.getString("EventType");

            // 4. 根据EventType分类处理
            if (eventJson.getString("processCode") != null) {
                switch (eventJson.getString("processCode")) {
                    case "PROC-EC6B5BD2-DE3C-456E-9591-6F7B377E39E4":
                        logger.info("采购付款申请（新） -> PROC-EC6B5BD2-DE3C-456E-9591-6F7B377E39E4");
//                        outSourceBill(eventJson);//采购付款申请（新）
                        break;
                    case "PROC-285CCF2B-524F-4055-BE62-FC31F490C654":
                        logger.info("采购付款申请（店群专用） -> PROC-285CCF2B-524F-4055-BE62-FC31F490C654");
                        agencyBill(eventJson);//采购付款申请（店群专用）
                        break;
                    case "PROC-AC33FC85-FB44-49FC-B926-1D966AE3BBD2":
                        logger.info("采购付款申请（新） -> PROC-AC33FC85-FB44-49FC-B926-1D966AE3BBD2");
//                        transferBill(eventJson);//特殊业务处理（新）
                        break;

                }
            }

            // 5. 返回success的加密数据
            Map<String, String> successMap = callbackCrypto.getEncryptedMap("success");
            logger.info("=============================================================================================================================钉钉回调执行结束《《《《《《《《《《《《《《《《《《《《《《《《《《《《《《《《《");
            return successMap;
        }catch (Exception e){
            logger.error("execute", e);
            Assert.state(false, "excute:{}", e);
        }
        return null;
    }

    /**
     * 获取钉钉应用token
     */
    public String getDingTalkToken() throws Exception {
        GetAccessTokenResponseBody tokenBody = DingTalkUtils.getTokenBody(APPKEY, AppSecret);
        return tokenBody.getAccessToken();
    }

    /**
     * 委外采购（付款单）->采购付款申请(新)
     * 回调，审核付款单
     */
    public void outSourceBill(JSONObject eventJson) throws Exception {
        //获取token
        String token = getDingTalkToken();
        //实例ID
        String processInstanceId = eventJson.getString("processInstanceId");
        //查询实例的具体信息
        GetProcessInstanceResponse processInstanceDetails = DingTalkUtils.getProcessInstanceDetails(token, processInstanceId);
        GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResult processInstanceResponseBodyResult = processInstanceDetails.getBody().getResult();
        //审批状态,该回调事件只监听，审批完成的
        if (processInstanceResponseBodyResult.getStatus().equals("RUNNING")) {//审批中
            logger.info("采购付款申请(新)--->审批中");
        } else if (processInstanceResponseBodyResult.getStatus().equals("TERMINATED")) {//已撤销
            logger.info("采购付款申请(新)--->已撤销");
        } else if (processInstanceResponseBodyResult.getStatus().equals("COMPLETED")) {//审批完成
            logger.info("采购付款申请(新)--->审批完成");
        }
        //获取所有审批意见
        List<GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultOperationRecords> operationRecords = processInstanceResponseBodyResult.getOperationRecords();

        //审批意见过滤掉：START_PROCESS_INSTANCE：发起流程实例
        operationRecords = operationRecords.stream().filter(item -> !item.getType().equals("START_PROCESS_INSTANCE")).collect(Collectors.toList());
    }

    /**
     * 代理采购（付款单）->采购付款申请(店群专用)
     * 回调，审核付款单
     */
    public void agencyBill(JSONObject eventJson) throws Exception{
        try {


            //获取token
            String token = getDingTalkToken();
            //实例ID
            String processInstanceId = eventJson.getString("processInstanceId");
            DingU8cBillEntity dingU8cBillEntity = queryU8CDing(processInstanceId);
            if (dingU8cBillEntity == null) {
                Assert.state(false,"代理采购（付款单）->采购付款申请(店群专用)回调：未查询到相关审批实例，实例id：{}",processInstanceId);
            }
            //查询实例的具体信息
            GetProcessInstanceResponse processInstanceDetails = DingTalkUtils.getProcessInstanceDetails(token, processInstanceId);
            GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResult processInstanceResponseBodyResult = processInstanceDetails.getBody().getResult();
            //审批状态,该回调事件只监听，审批完成的
            if (processInstanceResponseBodyResult.getStatus().equals("RUNNING")) {//审批中
                logger.info("采购付款申请(店群专用)--->审批中");
            } else if (processInstanceResponseBodyResult.getStatus().equals("TERMINATED")) {//已撤销
                logger.info("采购付款申请(店群专用)--->已撤销");
            } else if (processInstanceResponseBodyResult.getStatus().equals("COMPLETED")) {//审批完成
                logger.info("采购付款申请(店群专用)--->审批完成");
            }
            //获取所有审批意见
            List<GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultOperationRecords> operationRecords = processInstanceResponseBodyResult.getOperationRecords();

            //审批意见过滤掉：START_PROCESS_INSTANCE：发起流程实例
            operationRecords = operationRecords.stream().filter(item -> !item.getType().equals("START_PROCESS_INSTANCE")).collect(Collectors.toList());
            System.out.println(operationRecords);

            String remark="";

            //钉钉问题记录执行明细
            for (GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultOperationRecords operationRecord : operationRecords) {
                //操作结果
                switch (operationRecord.getResult()) {
                    case "AGREE":
                        dingU8cBillEntity.setDing_res("同意");
                        break;
                    case "REFUSE":
                        dingU8cBillEntity.setDing_res("拒绝");
                        break;
                    case "NONE":
                        dingU8cBillEntity.setDing_res("未处理");
                        break;
                }
                //处理意见
                remark = operationRecord.getRemark();
            }
            dingU8cBillDao.saveOrUpDateV2(dingU8cBillEntity);

            //审批u8c单据
            approveArapDjzb(dingU8cBillEntity,remark);
        }catch (Exception e){
            logger.error("代理采购（付款单）->采购付款申请(店群专用)回调报错："+e);
        }
    }

    /**
     * 根据ding_talk_process_id查询
     */
    public DingU8cBillEntity queryU8CDing(String ding_talk_process_id){
        DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity();
        dingU8cBillEntity.setDingTalkProcessId(ding_talk_process_id);
        List<DingU8cBillEntity> query = dingU8cBillDao.query(dingU8cBillEntity);
        if(query.size()==0){
            return null;
        }else{
            return query.get(0);
        }
    }

    /**
     * 审核付款单
     * @param remark 处理意见
     */
    public void approveArapDjzb(DingU8cBillEntity dingU8cBillEntity,String remark){
        ArapDjzb arapDjzb = new ArapDjzb();
    }
}
