package com.hzya.frame.plugin.lets.dingtalk.utils;

import cn.hutool.core.lang.Assert;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponse;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponseBody;
import com.aliyun.dingtalkstorage_1_0.models.AddPermissionResponse;
import com.aliyun.dingtalkstorage_1_0.models.CommitFileRequest;
import com.aliyun.dingtalkstorage_1_0.models.CommitFileResponse;
import com.aliyun.dingtalkstorage_1_0.models.GetFileUploadInfoResponse;
import com.aliyun.dingtalkworkflow_1_0.Client;
import com.aliyun.dingtalkworkflow_1_0.models.*;
import com.aliyun.tea.TeaException;
import com.aliyun.teautil.models.RuntimeOptions;
import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.*;
import com.dingtalk.api.response.*;
import com.taobao.api.ApiException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.List;

public class DingTalkUtils {
    protected static Logger logger = LogManager.getLogger(DingTalkUtils.class);

    private final static String PROTOCOL = "https";
    private final static String REGIONID = "central";
    public static com.aliyun.dingtalkworkflow_1_0.Client createClient() throws Exception {
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config();
        config.protocol = PROTOCOL;
        config.regionId = REGIONID;
        return new com.aliyun.dingtalkworkflow_1_0.Client(config);
    }


    /**
     * 企业内部应用
     * 获取钉钉access_token
     *
     * @return
     * @throws Exception
     */
    public static GetAccessTokenResponseBody getTokenBody(String appkey, String appsecret) throws Exception {
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config();
        config.protocol = PROTOCOL;
        config.regionId = REGIONID;
        com.aliyun.dingtalkoauth2_1_0.Client client = new com.aliyun.dingtalkoauth2_1_0.Client(config);
        com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenRequest getAccessTokenRequest = new com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenRequest()
                .setAppKey(appkey)//正式环境
                .setAppSecret(appsecret);//正式环境
        try {
            GetAccessTokenResponse accessToken = client.getAccessToken(getAccessTokenRequest);
            return accessToken.getBody();
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                logger.error(err);
            }

        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                logger.error(err);
            }
        }
        return null;
    }

    /**
     * 获取人员ID列表
     *
     * @param accessToken
     * @return
     */
    public static OapiUserListidResponse getUserIdList(String accessToken) {
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/user/listid");
            OapiUserListidRequest req = new OapiUserListidRequest();
            req.setDeptId(1L);
            OapiUserListidResponse rsp = client.execute(req, accessToken);
            System.out.println(rsp.getBody());
            return rsp;
        } catch (ApiException e) {
            e.printStackTrace();
            logger.error(e);
        }
        return null;
    }

    /**
     * 获取部门列表
     *
     * @param accessToken
     * @return
     */
    public static OapiV2DepartmentListsubResponse getDeptList(String accessToken, Long deptid) {
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/department/listsub");
            OapiV2DepartmentListsubRequest req = new OapiV2DepartmentListsubRequest();
            if (deptid != null) {
                req.setDeptId(deptid);
            }
            req.setLanguage("zh_CN");
            OapiV2DepartmentListsubResponse rsp = client.execute(req, accessToken);
            return rsp;
        } catch (ApiException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据部门ID获取部门详情
     *
     * @param accessToken
     * @param deptId
     * @return
     */
    public static OapiV2DepartmentGetResponse getDeptByDeptId(String accessToken, Long deptId) {
        OapiV2DepartmentGetResponse oapiV2DepartmentGetResponse = new OapiV2DepartmentGetResponse();
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/department/get");
            OapiV2DepartmentGetRequest req = new OapiV2DepartmentGetRequest();
            req.setDeptId(deptId);
            oapiV2DepartmentGetResponse = client.execute(req, accessToken);
        } catch (ApiException e) {
            e.printStackTrace();
            Assert.state(false, e.getMessage());
        }
        return oapiV2DepartmentGetResponse;
    }

    /**
     * 根据部门id查询用户id列表
     * 开发组：dept_id--》624820307L
     *
     * @param accessToken
     * @param deptId
     */
    public static OapiUserListidResponse getUserIdListByDeptId(String accessToken, Long deptId) {
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/user/listid");
            OapiUserListidRequest req = new OapiUserListidRequest();
            req.setDeptId(deptId);
            OapiUserListidResponse rsp = client.execute(req, accessToken);
            return rsp;
        } catch (ApiException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据用户id查询用户详情
     *
     * @param accessToken
     * @param userId
     * @return
     */
    public static OapiV2UserGetResponse getUserById(String accessToken, String userId) {
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/get");
            OapiV2UserGetRequest req = new OapiV2UserGetRequest();
            req.setUserid(userId);
            OapiV2UserGetResponse rsp = client.execute(req, accessToken);
            return rsp;
        } catch (ApiException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据手机号查询用户详情
     *
     * @param accessToken
     * @param mobile
     * @return
     */
    public static OapiV2UserGetbymobileResponse getUserByMobile(String accessToken, String mobile) {
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/getbymobile");
            OapiV2UserGetbymobileRequest req = new OapiV2UserGetbymobileRequest();
            req.setMobile(mobile);
            OapiV2UserGetbymobileResponse rsp = client.execute(req, accessToken);
            return rsp;
        } catch (ApiException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据部门id获取用户详细列表
     *
     * @return
     */
    public static OapiV2UserListResponse getUserDetailsByDeptId(String accessToken, Long deptId, Long cursor, Long size) {
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/list");
            OapiV2UserListRequest req = new OapiV2UserListRequest();
            req.setDeptId(deptId);
            req.setCursor(cursor);
            req.setSize(size);
            req.setLanguage("zh_CN");
            OapiV2UserListResponse rsp = client.execute(req, accessToken);
            return rsp;
        } catch (ApiException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据模板名称获取审批模板code
     *
     * @param accessToken
     * @param TemplateName
     * @return
     * @throws Exception
     */
    public static GetProcessCodeByNameResponse getProcessCode(String accessToken, String TemplateName) throws Exception {
        try {
            com.aliyun.dingtalkworkflow_1_0.Client client = createClient();
            GetProcessCodeByNameHeaders getProcessCodeByNameHeaders = new GetProcessCodeByNameHeaders();
            getProcessCodeByNameHeaders.xAcsDingtalkAccessToken = accessToken;
            GetProcessCodeByNameRequest getProcessCodeByNameRequest = new GetProcessCodeByNameRequest()
                    .setName(TemplateName);
            GetProcessCodeByNameResponse processCodeByNameWithOptions = client.getProcessCodeByNameWithOptions(getProcessCodeByNameRequest, getProcessCodeByNameHeaders, new RuntimeOptions());
            return processCodeByNameWithOptions;
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }

        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }
        }
        return null;
    }

    /**
     * 根据模板code获取表单的具体内容
     *
     * @param accessToken
     * @param processCode
     * @return
     * @throws Exception
     */
    public static QuerySchemaByProcessCodeResponse getBillSchema(String accessToken, String processCode) throws Exception {

        com.aliyun.dingtalkworkflow_1_0.Client client = createClient();
        QuerySchemaByProcessCodeHeaders querySchemaByProcessCodeHeaders = new QuerySchemaByProcessCodeHeaders();
        querySchemaByProcessCodeHeaders.xAcsDingtalkAccessToken = accessToken;
        QuerySchemaByProcessCodeRequest querySchemaByProcessCodeRequest = new QuerySchemaByProcessCodeRequest()
                .setProcessCode(processCode);
        try {
            QuerySchemaByProcessCodeResponse res = client.querySchemaByProcessCodeWithOptions(querySchemaByProcessCodeRequest, querySchemaByProcessCodeHeaders, new RuntimeOptions());
            return res;
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }
        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }
        }
        return null;

    }

    /**
     * 服务端APIOA审批官方OA审批审批实例发起审批实例
     * 发起审批实例
     *
     * @param accessToken             accessToken
     * @param originatorUserId        审批发起人的userId
     * @param processCode             审批流的唯一码,模板编码
     * @param deptId                  审批发起人所在的部门ID
     * @param microappAgentId         应用标识AgentId
     * @param approverList            不使用审批流模板时，直接指定的审批人列表，
     * @param formComponentValuesList 表单数据内容，控件列表，最大列表长度：150
     * @return
     * @throws Exception
     */
    public static StartProcessInstanceResponse InitiateApprovalInstance(String accessToken, String originatorUserId, String processCode, Long deptId, Long microappAgentId, List<StartProcessInstanceRequest.StartProcessInstanceRequestApprovers> approverList, List<StartProcessInstanceRequest.StartProcessInstanceRequestFormComponentValues> formComponentValuesList) throws Exception {
        com.aliyun.dingtalkworkflow_1_0.Client client = createClient();

        StartProcessInstanceHeaders startProcessInstanceHeaders = new StartProcessInstanceHeaders();
        startProcessInstanceHeaders.xAcsDingtalkAccessToken = accessToken;
        StartProcessInstanceRequest startProcessInstanceRequest = new StartProcessInstanceRequest()
                .setDeptId(deptId)
                .setApprovers(approverList)
                .setMicroappAgentId(microappAgentId)
                .setOriginatorUserId(originatorUserId)
                .setProcessCode(processCode)
                .setFormComponentValues(formComponentValuesList);
        try {
            StartProcessInstanceResponse startProcessInstanceResponse = client.startProcessInstanceWithOptions(startProcessInstanceRequest, startProcessInstanceHeaders, new RuntimeOptions());
            return startProcessInstanceResponse;
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
                Assert.state(false, err.getMessage());
            }

        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                // err 中含有 code 和 message 属性，可帮助开发定位问题
                Assert.state(false, err.getMessage());
            }

        }
        return null;

    }

    /**
     * 根据审批实例ID，获取该审批表单的详情内容
     *
     * @param accessToken
     * @param processInstanceId
     * @return
     * @throws Exception
     */
    public static GetProcessInstanceResponse getProcessInstanceDetails(String accessToken, String processInstanceId) throws Exception {
        com.aliyun.dingtalkworkflow_1_0.Client client = createClient();

        GetProcessInstanceHeaders getProcessInstanceHeaders = new GetProcessInstanceHeaders();
        getProcessInstanceHeaders.xAcsDingtalkAccessToken = accessToken;
        GetProcessInstanceRequest getProcessInstanceRequest = new GetProcessInstanceRequest()
                .setProcessInstanceId(processInstanceId);
        try {
            GetProcessInstanceResponse processInstanceWithOptions = client.getProcessInstanceWithOptions(getProcessInstanceRequest, getProcessInstanceHeaders, new RuntimeOptions());
            return processInstanceWithOptions;
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }
        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }
        }
        return null;
    }

    /**
     * 根据审批实例ID，撤销审批实例
     *
     * @param accessToken       accessToken
     * @param processInstanceId 审批实例ID
     * @param isSystem          是否通过系统操作。 - **true**：由系统直接终止 - **false**：由指定的操作者终止（需要传发起人才能撤销），妈的，这里有bug，不是发起人也能删除
     * @param remark            终止说明。
     * @param operatingUserId   发起人ID。
     * @throws Exception
     */
    public static TerminateProcessInstanceResponse revocationProcessInstance(String accessToken, String processInstanceId, boolean isSystem, String remark, String operatingUserId) throws Exception {

        TerminateProcessInstanceResponse terminateProcessInstanceResponse = new TerminateProcessInstanceResponse();
        com.aliyun.dingtalkworkflow_1_0.Client client = createClient();

        com.aliyun.dingtalkworkflow_1_0.models.TerminateProcessInstanceHeaders terminateProcessInstanceHeaders = new com.aliyun.dingtalkworkflow_1_0.models.TerminateProcessInstanceHeaders();
        terminateProcessInstanceHeaders.xAcsDingtalkAccessToken = accessToken;
        com.aliyun.dingtalkworkflow_1_0.models.TerminateProcessInstanceRequest terminateProcessInstanceRequest = new com.aliyun.dingtalkworkflow_1_0.models.TerminateProcessInstanceRequest()
                .setIsSystem(isSystem)
                .setProcessInstanceId(processInstanceId)
                .setRemark(remark)
                .setOperatingUserId(operatingUserId);

        try {
            terminateProcessInstanceResponse = client.terminateProcessInstanceWithOptions(terminateProcessInstanceRequest, terminateProcessInstanceHeaders, new RuntimeOptions());
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());

            }

        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());

            }
        }
        return terminateProcessInstanceResponse;


    }

    /**
     * 获取审批钉盘空间信息
     *
     * @param accessToken 获取审批钉盘空间信息
     * @param userId      用户的userId。
     */
    public static GetAttachmentSpaceResponse getProcessDingPanSpaceInfo(String accessToken, String userId) throws Exception {
        com.aliyun.dingtalkworkflow_1_0.Client client = createClient();

        com.aliyun.dingtalkworkflow_1_0.models.GetAttachmentSpaceHeaders getAttachmentSpaceHeaders = new com.aliyun.dingtalkworkflow_1_0.models.GetAttachmentSpaceHeaders();
        getAttachmentSpaceHeaders.xAcsDingtalkAccessToken = accessToken;
        com.aliyun.dingtalkworkflow_1_0.models.GetAttachmentSpaceRequest getAttachmentSpaceRequest = new com.aliyun.dingtalkworkflow_1_0.models.GetAttachmentSpaceRequest()
                .setUserId(userId);
        GetAttachmentSpaceResponse getAttachmentSpaceResponse = new GetAttachmentSpaceResponse();

        try {
            getAttachmentSpaceResponse = client.getAttachmentSpaceWithOptions(getAttachmentSpaceRequest, getAttachmentSpaceHeaders, new RuntimeOptions());
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }

        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }
        }
        return getAttachmentSpaceResponse;
    }

    /**
     * 获取文件上传信息
     *
     * @param accessToken token
     * @param spaceId     钉盘空间id
     * @param unionId     操作者ID
     * @param protocol    HEADER_SIGNATURE
     * @param multipart   是否需要分片上传。 - **true**：需要 - **false**：不需要 >- 5G以下文件，设为false，简化上传步骤。 >- 5G以上文件，必须设为true，否则会上传失败。
     */
    public static GetFileUploadInfoResponse getFileUploadInfo(String accessToken, String spaceId, String unionId, String protocol, Boolean multipart) throws Exception {
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config();
        config.protocol = PROTOCOL;
        config.regionId = REGIONID;
        com.aliyun.dingtalkstorage_1_0.Client client = new com.aliyun.dingtalkstorage_1_0.Client(config);
        GetFileUploadInfoResponse getFileUploadInfoResponse = new GetFileUploadInfoResponse();
        com.aliyun.dingtalkstorage_1_0.models.GetFileUploadInfoHeaders getFileUploadInfoHeaders = new com.aliyun.dingtalkstorage_1_0.models.GetFileUploadInfoHeaders();
        getFileUploadInfoHeaders.xAcsDingtalkAccessToken = accessToken;
        com.aliyun.dingtalkstorage_1_0.models.GetFileUploadInfoRequest getFileUploadInfoRequest = new com.aliyun.dingtalkstorage_1_0.models.GetFileUploadInfoRequest()
                .setProtocol(protocol)
                .setMultipart(multipart)
                .setUnionId(unionId);
        try {
            getFileUploadInfoResponse = client.getFileUploadInfoWithOptions(spaceId, getFileUploadInfoRequest, getFileUploadInfoHeaders, new RuntimeOptions());
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }

        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }

        }
        return getFileUploadInfoResponse;


    }

    /**
     * 提交文件
     *
     * @param accessToken                          token
     * @param spaceId                              空间ID
     * @param unionId                              操作者unionId
     * @param uploadKey                            添加文件唯一标识。获得文件上传信息是获得
     * @param fileName                             文件名称
     * @param parentId                             父目录Id。根目录时，该参数是0。，传FileManager中对应的id
     * @param commitFileRequestOptionAppProperties 当前文件的应用属性信息
     * @return
     * @throws Exception
     */
    public static CommitFileResponse submitFile(String accessToken, String spaceId, String unionId, String uploadKey, String fileName, String parentId, CommitFileRequest.CommitFileRequestOptionAppProperties commitFileRequestOptionAppProperties) throws Exception {
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config();
        config.protocol = PROTOCOL;
        config.regionId = REGIONID;
        com.aliyun.dingtalkstorage_1_0.Client client = new com.aliyun.dingtalkstorage_1_0.Client(config);

        CommitFileResponse commitFileResponse = new CommitFileResponse();

        com.aliyun.dingtalkstorage_1_0.models.CommitFileHeaders commitFileHeaders = new com.aliyun.dingtalkstorage_1_0.models.CommitFileHeaders();
        commitFileHeaders.xAcsDingtalkAccessToken = accessToken;

        com.aliyun.dingtalkstorage_1_0.models.CommitFileRequest.CommitFileRequestOption option = new com.aliyun.dingtalkstorage_1_0.models.CommitFileRequest.CommitFileRequestOption()
                .setAppProperties(java.util.Arrays.asList(
                        commitFileRequestOptionAppProperties
                ));
        com.aliyun.dingtalkstorage_1_0.models.CommitFileRequest commitFileRequest = new com.aliyun.dingtalkstorage_1_0.models.CommitFileRequest()
                .setName(fileName)
                .setUploadKey(uploadKey)
                .setParentId(parentId)
                .setOption(option)
                .setUnionId(unionId);
        try {
            commitFileResponse = client.commitFileWithOptions(spaceId, commitFileRequest, commitFileHeaders, new RuntimeOptions());
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }

        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }

        }
        return commitFileResponse;
    }

    /**
     * 根据用户的unionId添加权限，上传钉盘时用户总掉权限，导致上传失败，所以需要根据用户的unionId添加权限
     * 仅限审批实例中有附件时。
     * @param accessToken 调用企业接口凭证
     * @param spaceId 空间Id
     * @param unionId 操作用户的unionId
     * @return
     * @throws Exception
     */
    public static AddPermissionResponse addFileStorageByUnionId(String accessToken, String spaceId, String unionId) throws Exception {
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config();
        config.protocol = "https";
        config.regionId = "central";
        com.aliyun.dingtalkstorage_1_0.Client client = new com.aliyun.dingtalkstorage_1_0.Client(config);

        AddPermissionResponse addPermissionResponse = new AddPermissionResponse();

        com.aliyun.dingtalkstorage_1_0.models.AddPermissionHeaders addPermissionHeaders = new com.aliyun.dingtalkstorage_1_0.models.AddPermissionHeaders();
        addPermissionHeaders.xAcsDingtalkAccessToken = accessToken;
        com.aliyun.dingtalkstorage_1_0.models.AddPermissionRequest.AddPermissionRequestMembers members0 = new com.aliyun.dingtalkstorage_1_0.models.AddPermissionRequest.AddPermissionRequestMembers()
                .setType("USER")
                .setId(unionId);
        com.aliyun.dingtalkstorage_1_0.models.AddPermissionRequest addPermissionRequest = new com.aliyun.dingtalkstorage_1_0.models.AddPermissionRequest()
                .setUnionId(unionId)
                .setRoleId("MANAGER")
                .setMembers(java.util.Arrays.asList(
                        members0
                ));
        try {
            addPermissionResponse = client.addPermissionWithOptions(spaceId, "0", addPermissionRequest, addPermissionHeaders, new RuntimeOptions());
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }

        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }
        }
        return addPermissionResponse;
    }

    /**
     * /drive/spaces
     * 钉盘：新建空间
     * 使用场景：审批实例附件存放
     * @param token
     * @param spaceName
     * @param unionId
     */
    public static void addSpaces(String token,String spaceName,String unionId) throws Exception {
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config();
        config.protocol = "https";
        config.regionId = "central";
        com.aliyun.dingtalkdrive_1_0.Client client = new com.aliyun.dingtalkdrive_1_0.Client(config);

        com.aliyun.dingtalkdrive_1_0.models.AddSpaceHeaders addSpaceHeaders = new com.aliyun.dingtalkdrive_1_0.models.AddSpaceHeaders();
        addSpaceHeaders.xAcsDingtalkAccessToken = token;
        com.aliyun.dingtalkdrive_1_0.models.AddSpaceRequest addSpaceRequest = new com.aliyun.dingtalkdrive_1_0.models.AddSpaceRequest()
                .setName(spaceName)
                .setUnionId(unionId);
        try {
            client.addSpaceWithOptions(addSpaceRequest, addSpaceHeaders, new com.aliyun.teautil.models.RuntimeOptions());
        } catch (TeaException err) {
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }

        } catch (Exception _err) {
            TeaException err = new TeaException(_err.getMessage(), _err);
            if (!com.aliyun.teautil.Common.empty(err.code) && !com.aliyun.teautil.Common.empty(err.message)) {
                Assert.state(false, err.getMessage());
            }
        }
    }

}
